;+
;
; Ice freezing point
; ==================
;  fsfzpt: freezing point of seawater in degrees celsius
;       units : salinity        pfs       (ipss-78)
;               pressure        pfp      decibars
;               temperature     fszfpt   degrees celsius
;               freezing pt
;	reference : unesco tech. papers in the marine science no 28 1978
;               eighth report jpots
;               annex 6 freezing point of seawater F.J.Millero pp.29-35
;	checkvalue: fsfzpt=-2.588567 deg.c,for s=40.0,p=500 decibars
;
; @categories
;
; @param PFS {in}{required}
; salinity
;
; @param PFP {in}{required}
; pressure decibars
;
; @examples
;
; IDL> pfs=40.0
; IDL> pfp=500
; IDL> result=fsfzpt( pfs, pfp)
; IDL> print,result
;     -2.58857
;
; @returns
; temperature freezing pt degrees celsius
;
; @uses
;
; @restrictions
;
; @examples
;
; @history
;
; - fplod 20091208T163045Z zeus.locean-ipsl.upmc.fr (Linux)
;
;   * add example
;
; @version
; $Id$
;
; @todo seb
;
;-
FUNCTION fsfzpt, pfs, pfp
;
  compile_opt idl2, strictarrsubs
;
  RETURN, ( -0.0575 + 1.710523e-3 * sqrt(pfs) - 2.154996e-4 * pfs  ) * pfs - 7.53e-4 * pfp
END
